unit Receiverthread;

interface

uses
  dialogs,Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  ScktComp2,DCPrijndael,cyclicBuffer,decompressionThread, generic;

type
  TReceiverThread = class(TThread)
  private
   receiveCipher: TDCP_rijndael;
   rawReceiveBuffer: TCyclicBuffer;
   frmOwner: TForm;
   DecompressionThread: TDecompressionThread;
  protected
    procedure Execute; override;
  public
    constructor create(_frmOwner: TForm; _receiveCipher: TDCP_rijndael; _DecompressionThread: TDecompressionThread);
    destructor Destroy; override;
    procedure handleReceivedBuffer(Socket: TCustomWinSocket);
  end;

implementation

uses unit1;



{constructor}
constructor TReceiverThread.create(_frmOwner: TForm; _receiveCipher: TDCP_rijndael; _DecompressionThread: TDecompressionThread);
begin
   inherited create(true); // create but don't start running yet
   frmOwner := _frmOwner;
   receiveCipher := _receiveCipher;
   DecompressionThread := _DecompressionThread;
   rawReceiveBuffer := TCyclicBuffer.create(bufsize);
end;

{destructor}
destructor TReceiverThread.Destroy;
begin
   rawReceiveBuffer.free;
   inherited destroy;
end;


{main thread entry point}
procedure TReceiverThread.Execute;
var
   receiveCryptoBufferIn, receiveCryptoBufferOut: array[0..cryptoBufferSize-1] of byte;
   checksum, receivedChecksum: byte;
begin
   while (not terminated) and (connected) do
   begin
      try
         if rawReceiveBuffer.available = 0 then
         begin
            logToFile('Nothing in socket buffer');
            while rawReceiveBuffer.available = 0 do
            begin
               sleep(100);
               if (terminated) or (not connected) then exit;
            end;
         end;

         logToFile('Processing encrypted bytes: ' + inttostr(sizeof(receiveCryptoBufferIn)) + ' / ' + inttostr(rawReceiveBuffer.available) + ' encrypted bytes');

         // copy bytes from the socket receivebuffer into receiveCryptoBuffer
         rawReceiveBuffer.popBuffer(@receiveCryptoBufferIn[0],sizeof(receiveCryptoBufferIn));
         logBytes('encrypted bytes:', @receiveCryptoBufferIn, sizeof(receiveCryptoBufferIn));

         // decrypt the bytes in receiveCryptoBuffer
         receiveCipher.Decrypt(receiveCryptoBufferIn, receiveCryptoBufferOut, sizeof(receiveCryptoBufferIn));
         logBytes('decrypted bytes:', @receiveCryptoBufferOut, sizeof(receiveCryptoBufferIn));

         checksum := calcChecksum(@receiveCryptoBufferOut, sizeof(receiveCryptoBufferOut));
         receivedChecksum := rawReceiveBuffer.pop;
         logToFile('Calculated crypto checksum:' + inttostr(checksum) + ', received checksum:' + inttostr(receivedChecksum));
         assert(checksum = receivedChecksum);

         // copy bytes from the crypto buffer into compressedBuffer
         DecompressionThread.handleNewBytes(@receiveCryptoBufferOut[0], sizeof(receiveCryptoBufferOut));
         logToFile('Added ' + inttostr(sizeof(receiveCryptoBufferOut)) + ' bytes to compressed receive buffer');
      except
         on E: Exception do
         begin
            logToFile('Error in receiver thread: ' + E.message);
         end;
      end;
   end;
end ;


{called when the audio client socket reads some bytes sent by the peer}
procedure TReceiverThread.handleReceivedBuffer(Socket: TCustomWinSocket);
var
   tmpBuf: array[0..65534] of byte;
   bytesRead: integer;
begin
   bytesRead := Socket.ReceiveBuf(tmpBuf, sizeOf(tmpBuf));
   logToFile('Received ' + inttostr(bytesRead) + ' bytes of encrypted audio data');

   // dump bytes into raw receive buffer
   rawReceiveBuffer.pushBuffer(@tmpBuf[0], bytesRead);
   logToFile('Dumped bytes into raw receive buffer, now ' + inttostr(rawReceiveBuffer.available) + ' bytes available');
end;

end.

